package org.apache.bcel.classfile;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.TreeSet;
import org.apache.bcel.generic.Type;
import org.apache.bcel.util.BCELComparator;
import org.apache.bcel.util.ClassQueue;
import org.apache.bcel.util.Repository;
import org.apache.bcel.util.SyntheticRepository;
import org.apache.commons.lang3.ArrayUtils;

public class JavaClass extends AccessFlags implements Cloneable, Node, Comparable<JavaClass> {
   public static final JavaClass[] EMPTY_ARRAY = new JavaClass[0];
   public static final byte HEAP = 1;
   public static final byte FILE = 2;
   public static final byte ZIP = 3;
   private static final boolean debug = Boolean.getBoolean("JavaClass.debug");
   private static BCELComparator bcelComparator = new BCELComparator() {
      public boolean equals(Object o1, Object o2) {
         JavaClass THIS = (JavaClass)o1;
         JavaClass THAT = (JavaClass)o2;
         return Objects.equals(THIS.getClassName(), THAT.getClassName());
      }

      public int hashCode(Object o) {
         JavaClass THIS = (JavaClass)o;
         return THIS.getClassName().hashCode();
      }
   };
   private String fileName;
   private final String packageName;
   private String sourceFileName;
   private int classNameIndex;
   private int superclassNameIndex;
   private String className;
   private String superclassName;
   private int major;
   private int minor;
   private ConstantPool constantPool;
   private int[] interfaces;
   private String[] interfaceNames;
   private Field[] fields;
   private Method[] methods;
   private Attribute[] attributes;
   private AnnotationEntry[] annotations;
   private byte source;
   private boolean isAnonymous;
   private boolean isNested;
   private boolean computedNestedTypeStatus;
   private transient Repository repository;

   static void Debug(String str) {
      if (debug) {
         System.out.println(str);
      }

   }

   public static BCELComparator getComparator() {
      return bcelComparator;
   }

   private static String indent(Object obj) {
      StringTokenizer tokenizer = new StringTokenizer(obj.toString(), "\n");
      StringBuilder buf = new StringBuilder();

      while(tokenizer.hasMoreTokens()) {
         buf.append("\t").append(tokenizer.nextToken()).append("\n");
      }

      return buf.toString();
   }

   public static void setComparator(BCELComparator comparator) {
      bcelComparator = comparator;
   }

   public JavaClass(int classNameIndex, int superclassNameIndex, String fileName, int major, int minor, int accessFlags, ConstantPool constantPool, int[] interfaces, Field[] fields, Method[] methods, Attribute[] attributes) {
      this(classNameIndex, superclassNameIndex, fileName, major, minor, accessFlags, constantPool, interfaces, fields, methods, attributes, (byte)1);
   }

   public JavaClass(int classNameIndex, int superclassNameIndex, String fileName, int major, int minor, int accessFlags, ConstantPool constantPool, int[] interfaces, Field[] fields, Method[] methods, Attribute[] attributes, byte source) {
      super(accessFlags);
      this.sourceFileName = "<Unknown>";
      this.source = 1;
      this.repository = SyntheticRepository.getInstance();
      if (interfaces == null) {
         interfaces = ArrayUtils.EMPTY_INT_ARRAY;
      }

      if (attributes == null) {
         attributes = Attribute.EMPTY_ARRAY;
      }

      if (fields == null) {
         fields = Field.EMPTY_FIELD_ARRAY;
      }

      if (methods == null) {
         methods = Method.EMPTY_METHOD_ARRAY;
      }

      this.classNameIndex = classNameIndex;
      this.superclassNameIndex = superclassNameIndex;
      this.fileName = fileName;
      this.major = major;
      this.minor = minor;
      this.constantPool = constantPool;
      this.interfaces = interfaces;
      this.fields = fields;
      this.methods = methods;
      this.attributes = attributes;
      this.source = source;
      Attribute[] var13 = attributes;
      int i = attributes.length;

      for(int var15 = 0; var15 < i; ++var15) {
         Attribute attribute = var13[var15];
         if (attribute instanceof SourceFile) {
            this.sourceFileName = ((SourceFile)attribute).getSourceFileName();
            break;
         }
      }

      this.className = constantPool.getConstantString(classNameIndex, (byte)7);
      this.className = Utility.compactClassName(this.className, false);
      int index = this.className.lastIndexOf(46);
      if (index < 0) {
         this.packageName = "";
      } else {
         this.packageName = this.className.substring(0, index);
      }

      if (superclassNameIndex > 0) {
         this.superclassName = constantPool.getConstantString(superclassNameIndex, (byte)7);
         this.superclassName = Utility.compactClassName(this.superclassName, false);
      } else {
         this.superclassName = "java.lang.Object";
      }

      this.interfaceNames = new String[interfaces.length];

      for(i = 0; i < interfaces.length; ++i) {
         String str = constantPool.getConstantString(interfaces[i], (byte)7);
         this.interfaceNames[i] = Utility.compactClassName(str, false);
      }

   }

   public void accept(Visitor v) {
      v.visitJavaClass(this);
   }

   public int compareTo(JavaClass obj) {
      return this.getClassName().compareTo(obj.getClassName());
   }

   private void computeNestedTypeStatus() {
      if (!this.computedNestedTypeStatus) {
         Attribute[] var1 = this.attributes;
         int var2 = var1.length;

         for(int var3 = 0; var3 < var2; ++var3) {
            Attribute attribute = var1[var3];
            if (attribute instanceof InnerClasses) {
               ((InnerClasses)attribute).forEach((innerClass) -> {
                  boolean innerClassAttributeRefersToMe = false;
                  String innerClassName = this.constantPool.getConstantString(innerClass.getInnerClassIndex(), (byte)7);
                  innerClassName = Utility.compactClassName(innerClassName, false);
                  if (innerClassName.equals(this.getClassName())) {
                     innerClassAttributeRefersToMe = true;
                  }

                  if (innerClassAttributeRefersToMe) {
                     this.isNested = true;
                     if (innerClass.getInnerNameIndex() == 0) {
                        this.isAnonymous = true;
                     }
                  }

               });
            }
         }

         this.computedNestedTypeStatus = true;
      }
   }

   public JavaClass copy() {
      try {
         JavaClass c = (JavaClass)this.clone();
         c.constantPool = this.constantPool.copy();
         c.interfaces = (int[])this.interfaces.clone();
         c.interfaceNames = (String[])this.interfaceNames.clone();
         c.fields = new Field[this.fields.length];
         Arrays.setAll(c.fields, (i) -> {
            return this.fields[i].copy(c.constantPool);
         });
         c.methods = new Method[this.methods.length];
         Arrays.setAll(c.methods, (i) -> {
            return this.methods[i].copy(c.constantPool);
         });
         c.attributes = new Attribute[this.attributes.length];
         Arrays.setAll(c.attributes, (i) -> {
            return this.attributes[i].copy(c.constantPool);
         });
         return c;
      } catch (CloneNotSupportedException var2) {
         return null;
      }
   }

   public void dump(DataOutputStream file) throws IOException {
      file.writeInt(-889275714);
      file.writeShort(this.minor);
      file.writeShort(this.major);
      this.constantPool.dump(file);
      file.writeShort(super.getAccessFlags());
      file.writeShort(this.classNameIndex);
      file.writeShort(this.superclassNameIndex);
      file.writeShort(this.interfaces.length);
      int[] var2 = this.interfaces;
      int var3 = var2.length;

      int var4;
      for(var4 = 0; var4 < var3; ++var4) {
         int interface1 = var2[var4];
         file.writeShort(interface1);
      }

      file.writeShort(this.fields.length);
      Field[] var6 = this.fields;
      var3 = var6.length;

      for(var4 = 0; var4 < var3; ++var4) {
         Field field = var6[var4];
         field.dump(file);
      }

      file.writeShort(this.methods.length);
      Method[] var7 = this.methods;
      var3 = var7.length;

      for(var4 = 0; var4 < var3; ++var4) {
         Method method = var7[var4];
         method.dump(file);
      }

      if (this.attributes != null) {
         file.writeShort(this.attributes.length);
         Attribute[] var8 = this.attributes;
         var3 = var8.length;

         for(var4 = 0; var4 < var3; ++var4) {
            Attribute attribute = var8[var4];
            attribute.dump(file);
         }
      } else {
         file.writeShort(0);
      }

      file.flush();
   }

   public void dump(File file) throws IOException {
      String parent = file.getParent();
      if (parent != null) {
         File dir = new File(parent);
         if (!dir.mkdirs() && !dir.isDirectory()) {
            throw new IOException("Could not create the directory " + dir);
         }
      }

      DataOutputStream dos = new DataOutputStream(new FileOutputStream(file));
      Throwable var4 = null;

      try {
         this.dump(dos);
      } catch (Throwable var13) {
         var4 = var13;
         throw var13;
      } finally {
         if (dos != null) {
            if (var4 != null) {
               try {
                  dos.close();
               } catch (Throwable var12) {
                  var4.addSuppressed(var12);
               }
            } else {
               dos.close();
            }
         }

      }

   }

   public void dump(OutputStream file) throws IOException {
      this.dump(new DataOutputStream(file));
   }

   public void dump(String fileName) throws IOException {
      this.dump(new File(fileName));
   }

   public boolean equals(Object obj) {
      return bcelComparator.equals(this, obj);
   }

   public JavaClass[] getAllInterfaces() throws ClassNotFoundException {
      ClassQueue queue = new ClassQueue();
      Set<JavaClass> allInterfaces = new TreeSet();
      queue.enqueue(this);

      while(!queue.empty()) {
         JavaClass clazz = queue.dequeue();
         JavaClass souper = clazz.getSuperClass();
         JavaClass[] interfaces = clazz.getInterfaces();
         if (clazz.isInterface()) {
            allInterfaces.add(clazz);
         } else if (souper != null) {
            queue.enqueue(souper);
         }

         JavaClass[] var6 = interfaces;
         int var7 = interfaces.length;

         for(int var8 = 0; var8 < var7; ++var8) {
            JavaClass iface = var6[var8];
            queue.enqueue(iface);
         }
      }

      return (JavaClass[])allInterfaces.toArray(EMPTY_ARRAY);
   }

   public AnnotationEntry[] getAnnotationEntries() {
      if (this.annotations == null) {
         this.annotations = AnnotationEntry.createAnnotationEntries(this.getAttributes());
      }

      return this.annotations;
   }

   public Attribute[] getAttributes() {
      return this.attributes;
   }

   public byte[] getBytes() {
      ByteArrayOutputStream baos = new ByteArrayOutputStream();

      try {
         DataOutputStream dos = new DataOutputStream(baos);
         Throwable var3 = null;

         try {
            this.dump(dos);
         } catch (Throwable var13) {
            var3 = var13;
            throw var13;
         } finally {
            if (dos != null) {
               if (var3 != null) {
                  try {
                     dos.close();
                  } catch (Throwable var12) {
                     var3.addSuppressed(var12);
                  }
               } else {
                  dos.close();
               }
            }

         }
      } catch (IOException var15) {
         IOException e = var15;
         e.printStackTrace();
      }

      return baos.toByteArray();
   }

   public String getClassName() {
      return this.className;
   }

   public int getClassNameIndex() {
      return this.classNameIndex;
   }

   public ConstantPool getConstantPool() {
      return this.constantPool;
   }

   public Field[] getFields() {
      return this.fields;
   }

   public String getFileName() {
      return this.fileName;
   }

   public int[] getInterfaceIndices() {
      return this.interfaces;
   }

   public String[] getInterfaceNames() {
      return this.interfaceNames;
   }

   public JavaClass[] getInterfaces() throws ClassNotFoundException {
      String[] interfaces = this.getInterfaceNames();
      JavaClass[] classes = new JavaClass[interfaces.length];

      for(int i = 0; i < interfaces.length; ++i) {
         classes[i] = this.repository.loadClass(interfaces[i]);
      }

      return classes;
   }

   public int getMajor() {
      return this.major;
   }

   public Method getMethod(java.lang.reflect.Method m) {
      Method[] var2 = this.methods;
      int var3 = var2.length;

      for(int var4 = 0; var4 < var3; ++var4) {
         Method method = var2[var4];
         if (m.getName().equals(method.getName()) && m.getModifiers() == method.getModifiers() && Type.getSignature(m).equals(method.getSignature())) {
            return method;
         }
      }

      return null;
   }

   public Method[] getMethods() {
      return this.methods;
   }

   public int getMinor() {
      return this.minor;
   }

   public String getPackageName() {
      return this.packageName;
   }

   public Repository getRepository() {
      return this.repository;
   }

   public final byte getSource() {
      return this.source;
   }

   public String getSourceFileName() {
      return this.sourceFileName;
   }

   public JavaClass getSuperClass() throws ClassNotFoundException {
      return "java.lang.Object".equals(this.getClassName()) ? null : this.repository.loadClass(this.getSuperclassName());
   }

   public JavaClass[] getSuperClasses() throws ClassNotFoundException {
      JavaClass clazz = this;
      List<JavaClass> allSuperClasses = new ArrayList();

      for(clazz = clazz.getSuperClass(); clazz != null; clazz = clazz.getSuperClass()) {
         allSuperClasses.add(clazz);
      }

      return (JavaClass[])allSuperClasses.toArray(EMPTY_ARRAY);
   }

   public String getSuperclassName() {
      return this.superclassName;
   }

   public int getSuperclassNameIndex() {
      return this.superclassNameIndex;
   }

   public int hashCode() {
      return bcelComparator.hashCode(this);
   }

   public boolean implementationOf(JavaClass inter) throws ClassNotFoundException {
      if (!inter.isInterface()) {
         throw new IllegalArgumentException(inter.getClassName() + " is no interface");
      } else if (this.equals(inter)) {
         return true;
      } else {
         JavaClass[] superInterfaces = this.getAllInterfaces();
         JavaClass[] var3 = superInterfaces;
         int var4 = superInterfaces.length;

         for(int var5 = 0; var5 < var4; ++var5) {
            JavaClass superInterface = var3[var5];
            if (superInterface.equals(inter)) {
               return true;
            }
         }

         return false;
      }
   }

   public final boolean instanceOf(JavaClass superclass) throws ClassNotFoundException {
      if (this.equals(superclass)) {
         return true;
      } else {
         JavaClass[] var2 = this.getSuperClasses();
         int var3 = var2.length;

         for(int var4 = 0; var4 < var3; ++var4) {
            JavaClass clazz = var2[var4];
            if (clazz.equals(superclass)) {
               return true;
            }
         }

         if (superclass.isInterface()) {
            return this.implementationOf(superclass);
         } else {
            return false;
         }
      }
   }

   public final boolean isAnonymous() {
      this.computeNestedTypeStatus();
      return this.isAnonymous;
   }

   public final boolean isClass() {
      return (super.getAccessFlags() & 512) == 0;
   }

   public final boolean isNested() {
      this.computeNestedTypeStatus();
      return this.isNested;
   }

   public final boolean isSuper() {
      return (super.getAccessFlags() & 32) != 0;
   }

   public void setAttributes(Attribute[] attributes) {
      this.attributes = attributes;
   }

   public void setClassName(String className) {
      this.className = className;
   }

   public void setClassNameIndex(int classNameIndex) {
      this.classNameIndex = classNameIndex;
   }

   public void setConstantPool(ConstantPool constantPool) {
      this.constantPool = constantPool;
   }

   public void setFields(Field[] fields) {
      this.fields = fields;
   }

   public void setFileName(String fileName) {
      this.fileName = fileName;
   }

   public void setInterfaceNames(String[] interfaceNames) {
      this.interfaceNames = interfaceNames;
   }

   public void setInterfaces(int[] interfaces) {
      this.interfaces = interfaces;
   }

   public void setMajor(int major) {
      this.major = major;
   }

   public void setMethods(Method[] methods) {
      this.methods = methods;
   }

   public void setMinor(int minor) {
      this.minor = minor;
   }

   public void setRepository(Repository repository) {
      this.repository = repository;
   }

   public void setSourceFileName(String sourceFileName) {
      this.sourceFileName = sourceFileName;
   }

   public void setSuperclassName(String superclassName) {
      this.superclassName = superclassName;
   }

   public void setSuperclassNameIndex(int superclassNameIndex) {
      this.superclassNameIndex = superclassNameIndex;
   }

   public String toString() {
      String access = Utility.accessToString(super.getAccessFlags(), true);
      access = access.isEmpty() ? "" : access + " ";
      StringBuilder buf = new StringBuilder(128);
      buf.append(access).append(Utility.classOrInterface(super.getAccessFlags())).append(" ").append(this.className).append(" extends ").append(Utility.compactClassName(this.superclassName, false)).append('\n');
      int size = this.interfaces.length;
      if (size > 0) {
         buf.append("implements\t\t");

         for(int i = 0; i < size; ++i) {
            buf.append(this.interfaceNames[i]);
            if (i < size - 1) {
               buf.append(", ");
            }
         }

         buf.append('\n');
      }

      buf.append("file name\t\t").append(this.fileName).append('\n');
      buf.append("compiled from\t\t").append(this.sourceFileName).append('\n');
      buf.append("compiler version\t").append(this.major).append(".").append(this.minor).append('\n');
      buf.append("access flags\t\t").append(super.getAccessFlags()).append('\n');
      buf.append("constant pool\t\t").append(this.constantPool.getLength()).append(" entries\n");
      buf.append("ACC_SUPER flag\t\t").append(this.isSuper()).append("\n");
      int var6;
      if (this.attributes.length > 0) {
         buf.append("\nAttribute(s):\n");
         Attribute[] var9 = this.attributes;
         int var5 = var9.length;

         for(var6 = 0; var6 < var5; ++var6) {
            Attribute attribute = var9[var6];
            buf.append(indent(attribute));
         }
      }

      AnnotationEntry[] annotations = this.getAnnotationEntries();
      int var14;
      if (annotations != null && annotations.length > 0) {
         buf.append("\nAnnotation(s):\n");
         AnnotationEntry[] var11 = annotations;
         var6 = annotations.length;

         for(var14 = 0; var14 < var6; ++var14) {
            AnnotationEntry annotation = var11[var14];
            buf.append(indent(annotation));
         }
      }

      if (this.fields.length > 0) {
         buf.append("\n").append(this.fields.length).append(" fields:\n");
         Field[] var12 = this.fields;
         var6 = var12.length;

         for(var14 = 0; var14 < var6; ++var14) {
            Field field = var12[var14];
            buf.append("\t").append(field).append('\n');
         }
      }

      if (this.methods.length > 0) {
         buf.append("\n").append(this.methods.length).append(" methods:\n");
         Method[] var13 = this.methods;
         var6 = var13.length;

         for(var14 = 0; var14 < var6; ++var14) {
            Method method = var13[var14];
            buf.append("\t").append(method).append('\n');
         }
      }

      return buf.toString();
   }
}
